%define	chroot_prefix	%{_var}/named-chroot

Name:		bind-altchroot
Version:	1.0
Release:	4%{?dist}

Summary:	An alternative chroot runtime environment for the ISC BIND DNS server, named
Group:		System Environment/Daemons
License:	Freely redistributable without restriction
URL:		http://pkgrepo.linuxtech.net/el6/
Prefix:		%{chroot_prefix}
Requires:	bind
Requires(pre):	sed
Requires(post):	grep
Requires(preun): grep, sed
Requires(postun): sed
Conflicts:	bind-chroot


%description
This package contains an alternative tree of files which can be used as a
chroot jail for the named DNS daemon from the BIND package.
This chroot tree is located in /var/named-chroot/ to avoid mounting loops.
To use it set 'ROOTDIR=/var/named-chroot' in '/etc/sysconfig/named'.


%prep


%build


%install
rm -rf %{buildroot}
mkdir -p %{buildroot}/%{chroot_prefix}/{dev,etc,var}
mkdir -p %{buildroot}/%{chroot_prefix}/var/{log,named,run/named,tmp}
mkdir -p %{buildroot}/%{chroot_prefix}/etc/{pki/dnssec-keys,named}
mkdir -p %{buildroot}/%{chroot_prefix}/%{_libdir}/bind
# these are required to prevent them being erased during upgrade of previous
# versions that included them (bug #130121):
touch %{buildroot}/%{chroot_prefix}/dev/null
touch %{buildroot}/%{chroot_prefix}/dev/random
touch %{buildroot}/%{chroot_prefix}/dev/zero
touch %{buildroot}/%{chroot_prefix}/etc/localtime
touch %{buildroot}/%{chroot_prefix}/etc/named.conf


%clean
rm -rf %{buildroot}


%pre
if [ "$1" -gt 0 ]; then
  if [ -f /etc/selinux/targeted/contexts/files/file_contexts.local ]; then
    # purge any old SELinux file settings for named chroot tree
    sed -i -e '/^\/var\/named-chroot.*/d' /etc/selinux/targeted/contexts/files/file_contexts.local
  fi
  if [ -d /etc/selinux/targeted/contexts/files ]; then
    # add SELinux file settings for named chroot tree
    cat >> /etc/selinux/targeted/contexts/files/file_contexts.local <<EOF
/var/named-chroot(/.*)? system_u:object_r:named_conf_t:s0
/var/named-chroot/etc(/.*)?     system_u:object_r:etc_t:s0
/var/named-chroot/proc(/.*)?    <<none>>
/var/named-chroot/etc/pki(/.*)? system_u:object_r:cert_t:s0
/var/named-chroot/var/tmp(/.*)? system_u:object_r:named_cache_t:s0
/var/named-chroot/var/named(/.*)?       system_u:object_r:named_zone_t:s0
/var/named-chroot/var/run/dbus(/.*)?    system_u:object_r:system_dbusd_var_run_t:s0
/var/named-chroot/var/run/named.*       system_u:object_r:named_var_run_t:s0
/var/named-chroot/var/log/named.*       --      system_u:object_r:named_log_t:s0
/var/named-chroot/var/named/data(/.*)?  system_u:object_r:named_cache_t:s0
/var/named-chroot/var/named/slaves(/.*)?        system_u:object_r:named_cache_t:s0
/var/named-chroot/var/named/dynamic(/.*)?       system_u:object_r:named_cache_t:s0
/var/named-chroot/etc/named\.rfc1912.zones      --      system_u:object_r:named_conf_t:s0
/var/named-chroot/dev   -d      system_u:object_r:device_t:s0
/var/named-chroot/var/log       -d      system_u:object_r:var_log_t:s0
/var/named-chroot/dev/log       -s      system_u:object_r:devlog_t:s0
/var/named-chroot/dev/null      -c      system_u:object_r:null_device_t:s0
/var/named-chroot/dev/zero      -c      system_u:object_r:zero_device_t:s0
/var/named-chroot/dev/random    -c      system_u:object_r:random_device_t:s0
/var/named-chroot/etc/rndc\.key --      system_u:object_r:dnssec_t:s0
/var/named-chroot/etc/named\.conf       --      system_u:object_r:named_conf_t:s0
/var/named-chroot/var/named/named\.ca   --      system_u:object_r:named_conf_t:s0
/var/named-chroot/etc/named\.root\.hints        --      system_u:object_r:named_conf_t:s0
/var/named-chroot/etc/named\.caching-nameserver\.conf   --      system_u:object_r:named_conf_t:s0
EOF
  fi
fi;
:;


%post
if [ "$1" -gt 0 ]; then
  [ -e %{chroot_prefix}/dev/random ] || \
    /bin/mknod %{chroot_prefix}/dev/random c 1 8
  [ -e %{chroot_prefix}/dev/zero ] || \
    /bin/mknod %{chroot_prefix}/dev/zero c 1 5
  [ -e %{chroot_prefix}/dev/null ] || \
    /bin/mknod %{chroot_prefix}/dev/null c 1 3
  rm -f %{chroot_prefix}/etc/localtime
  cp /etc/localtime %{chroot_prefix}/etc/localtime
  if ! grep -q '^ROOTDIR=' /etc/sysconfig/named; then
    echo 'ROOTDIR=/var/named-chroot' >> /etc/sysconfig/named
    /sbin/service named try-restart > /dev/null 2>&1 || :;
  fi
fi;
:;


%posttrans
if [ -x /usr/sbin/selinuxenabled ] && /usr/sbin/selinuxenabled; then
  [ -x /sbin/restorecon ] && /sbin/restorecon -R %{chroot_prefix} > /dev/null 2>&1;
fi;
:;


%preun
if [ "$1" -eq 0 ]; then
  rm -f %{chroot_prefix}/dev/{random,zero,null}
  rm -f %{chroot_prefix}/etc/localtime
  if grep -q '^ROOTDIR=' /etc/sysconfig/named; then
    # NOTE: Do NOT call `service named try-restart` because chroot
    # files will remain mounted.
    START=no
    [ -e /var/lock/subsys/named ] && START=yes
    /sbin/service named stop > /dev/null 2>&1 || :;
    sed -i -e '/^ROOTDIR=.*/d' /etc/sysconfig/named
    if [ "x$START" = xyes ]; then
      /sbin/service named start > /dev/null 2>&1 || :;
    fi
  fi
fi;
:;


%postun
if [ "$1" -eq 0 ]; then
  if [ -f /etc/selinux/targeted/contexts/files/file_contexts.local ]; then
    # purge all SELinux file settings for named chroot tree
    sed -i -e '/^\/var\/named-chroot.*/d' /etc/selinux/targeted/contexts/files/file_contexts.local
  fi
  if [ -x /usr/sbin/selinuxenabled ] && /usr/sbin/selinuxenabled; then
    [ -x /sbin/restorecon ] && /sbin/restorecon -R %{chroot_prefix} > /dev/null 2>&1;
  fi
fi;
:;


%files
%defattr(-,root,root,-)
%ghost %{chroot_prefix}/dev/null
%ghost %{chroot_prefix}/dev/random
%ghost %{chroot_prefix}/dev/zero
%ghost %{chroot_prefix}/etc/localtime
%defattr(0640,root,named,0750)
%dir %{chroot_prefix}
%dir %{chroot_prefix}/dev
%dir %{chroot_prefix}/etc
%dir %{chroot_prefix}/etc/named
%dir %{chroot_prefix}/etc/pki
%dir %{chroot_prefix}/etc/pki/dnssec-keys
%dir %{chroot_prefix}/var
%dir %{chroot_prefix}/var/run
%dir %{chroot_prefix}/var/named
%dir %{chroot_prefix}/usr
%dir %{chroot_prefix}/%{_libdir}
%dir %{chroot_prefix}/%{_libdir}/bind
%ghost %config(noreplace) %{chroot_prefix}/etc/named.conf
%defattr(0660,named,named,0770)
%dir %{chroot_prefix}/var/run/named
%dir %{chroot_prefix}/var/tmp
%dir %{chroot_prefix}/var/log


%changelog
* Mon Feb 13 2012 LTN Packager <packager-el6rpms@LinuxTECH.NET> - 1.0-4
- added SELinux file permission settings
- reported and fixed RHB #789886

* Sun Feb 12 2012 LTN Packager <packager-el6rpms@LinuxTECH.NET> - 1.0-1
- initial package version

